@@ -45,12 +45,12 @@ require Exporter;
 use strict;
 use POSIX qw(floor);
 
-use File::Path 2.08 qw(make_path remove_tree);
+use File::Path qw(make_path remove_tree);
 
 use canu::Defaults;
 use canu::Execution;
 use canu::Gatekeeper;
-use canu::Grid_Cloud;
+use canu::HTML;
 
 #  Map long reads to long reads with mhap.
 
@@ -60,28 +60,26 @@ use canu::Grid_Cloud;
 
 
 
-sub mhapConfigure ($$$) {
+sub mhapConfigure ($$$$) {
+    my $WRK     = shift @_;  #  Root work directory (the -d option to canu)
+    my $wrk     = $WRK;      #  Local work directory
     my $asm     = shift @_;
     my $tag     = shift @_;
     my $typ     = shift @_;
     my $bin     = getBinDirectory();
 
-    my $base;                #  e.g., $base/1-overlapper/mhap.sh
-    my $path;                #  e.g., $path/mhap.sh
+    $wrk = "$wrk/correction"  if ($tag eq "cor");
+    $wrk = "$wrk/trimming"    if ($tag eq "obt");
+    $wrk = "$wrk/unitigging"  if ($tag eq "utg");
 
-    $base = "correction"  if ($tag eq "cor");
-    $base = "trimming"    if ($tag eq "obt");
-    $base = "unitigging"  if ($tag eq "utg");
-
-    $path = "$base/1-overlapper";
+    my $path = "$wrk/1-overlapper";
 
     caFailure("invalid type '$typ'", undef)  if (($typ ne "partial") && ($typ ne "normal"));
 
-    goto allDone   if (skipStage($asm, "$tag-mhapConfigure") == 1);
-    goto allDone   if (fileExists("$path/precompute.sh")) && (fileExists("$path/mhap.sh"));
-    goto allDone   if (fileExists("$path/ovljob.files"));
-    goto allDone   if (-e "$base/$asm.ovlStore");
-    goto allDone   if (fileExists("$base/$asm.ovlStore.tar"));
+    goto allDone   if (skipStage($WRK, $asm, "$tag-mhapConfigure") == 1);
+    goto allDone   if (-e "$path/precompute.sh") && (-e "$path/mhap.sh");
+    goto allDone   if (-e "$path/ovljob.files");
+    goto allDone   if (-e "$wrk/$asm.ovlStore");
 
     print STDERR "--\n";
     print STDERR "-- OVERLAPPER (mhap) (correction)\n"  if ($tag eq "cor");
@@ -89,14 +87,14 @@ sub mhapConfigure ($$$) {
     print STDERR "-- OVERLAPPER (mhap) (assembly)\n"    if ($tag eq "utg");
     print STDERR "--\n";
 
-    make_path($path) if (! -d $path);
+    make_path("$path") if (! -d "$path");
 
-    #  Mhap parameters
+    #  Mhap parameters - filterThreshold needs to be a string, else it is printed as 5e-06.
 
     my ($numHashes, $minNumMatches, $threshold, $ordSketch, $ordSketchMer);
 
     if (!defined(getGlobal("${tag}MhapSensitivity"))) {
-        my $cov = getExpectedCoverage($tag, $asm);
+        my $cov = getExpectedCoverage($wrk, $asm);
 
         setGlobal("${tag}MhapSensitivity", "low");                          #  Yup, super inefficient.  The code is
         setGlobal("${tag}MhapSensitivity", "normal")   if ($cov <  60);     #  compact and clear and runs once.
@@ -130,36 +128,27 @@ sub mhapConfigure ($$$) {
         caFailure("invalid ${tag}MhapSensitivity=" . getGlobal("${tag}MhapSensitivity"), undef);
     }
 
-    # quick guess parameter adjustment for corrected reads, hack for now and should better take error rate into account
-    if (($tag eq "obt") || ($tag eq "utg")) {
-       $numHashes      = "128";
-       $minNumMatches  = 5;
-       $ordSketch      = 1000;
-       $threshold      = 1-getGlobal("${tag}OvlErrorRate");
-    }
-
-    # due to systematic bias in nanopore data, adjust threshold up by 5%
-    my $numNanoporeRaw = 0;
-    open(L, "< $base/$asm.gkpStore/libraries.txt") or caExit("can't open '$base/$asm.gkpStore/libraries.txt' for reading: $!", undef);
-    while (<L>) {
-        $numNanoporeRaw++         if (m/nanopore-raw/);
-    }
-    close(L);
-
-    $threshold += 0.05   if ($numNanoporeRaw > 0);
-
     my $filterThreshold = getGlobal("${tag}MhapFilterThreshold");
 
     #  Constants.
 
     my $merSize       = getGlobal("${tag}MhapMerSize");
 
-    my $numReads      = getNumberOfReadsEarliestVersion($asm);   #  Need to iterate over all read IDs!
+    my $numReads      = getNumberOfReadsInStore($wrk, $asm);
     my $memorySize    = getGlobal("${tag}mhapMemory");
-    my $blockPerGb    = getGlobal("${tag}MhapBlockSize") / (($numHashes < 768) ? 1 : 2);
+    my $blockPerGb    = getGlobal("${tag}MhapBlockSize");
+    if ($numHashes >= 768) {
+       $blockPerGb = int($blockPerGb / 2);
+    }
 
-    my $javaPath      = getGlobal("java");
-    my $javaMemory    = int(getGlobal("${tag}mhapMemory") * 1024 + 0.5);
+    # quick guess parameter adjustment for corrected reads, hack for now and should better take error rate into account
+    if (($tag eq "obt") || ($tag eq "utg")) {
+       $numHashes     /= 4;
+       $minNumMatches  = floor(1.5 * $minNumMatches);
+       $ordSketch      = floor($ordSketch / 2);
+       $threshold      = 1-getGlobal("${tag}OvlErrorRate");
+       $blockPerGb    *= 2;
+    }
 
     print STDERR "--\n";
     print STDERR "-- PARAMETERS: hashes=$numHashes, minMatches=$minNumMatches, threshold=$threshold\n";
@@ -178,12 +167,14 @@ sub mhapConfigure ($$$) {
 
     my @hashes;    #  One for each job, the block that is the hash table
     my @skipSelf;  #  One for each job, jobs that would search block N against hash block N need to be handled special
+    my @convert;   #  One for each job, flags to the mhap-ovl conversion program
 
     push @blocks,   "no zeroth block, makes the loop where this is used easier";
     push @blockBgn, "no zeroth block";
     push @blockLen, "no zeroth block";
     push @hashes,   "no zeroth job";
     push @skipSelf, "no zeroth job";
+    push @convert,  "no zeroth job";
 
     for (my $bgn=1; $bgn < $numReads; $bgn += $blockSize) {
         my $end = $bgn + $blockSize - 1;
@@ -269,7 +260,7 @@ sub mhapConfigure ($$$) {
                 for (my $qid=$qbgn; $qid <= $qend; $qid++) {
                     my $qry = substr("000000" . $qid, -6);             #  Name for the query block
 
-                    symlink("../../blocks/$qry.dat", "$path/queries/$job/$qry.dat");
+                    symlink("$path/blocks/$qry.dat", "$path/queries/$job/$qry.dat");
                 }
 
             } else {
@@ -280,36 +271,41 @@ sub mhapConfigure ($$$) {
             #  This is easy, the ID of the hash.
 
             push @hashes, substr("000000" . $bid, -6);  #  One new job for block bid with qend-qbgn query files in it
+
+            #  Annoyingly, if we're against 'self', then the conversion needs to know that the query IDs
+            #  aren't offset by the number of hash reads.
+
+            if ($andSelf eq "") {
+                push @convert, "-h $blockBgn[$bid] $blockLen[$bid] -q $blockBgn[$qbgn]";
+            } else {
+                push @convert, "-h $blockBgn[$bid] 0 -q $blockBgn[$bid]";
+            }
         }
     }
 
     close(L);
 
-    #  Tar up the queries directory.  Only useful for cloud support.
+    #  The ignore file is created in Meryl.pm
 
-    runCommandSilently($path, "tar -cf queries.tar queries", 1);
-    stashFile("$path/queries.tar");
 
     #  Create a script to generate precomputed blocks, including extracting the reads from gkpStore.
 
-    #OPTIMIZE
-    #OPTIMIZE  Probably a big optimization for cloud assemblies, the block fasta inputs can be
-    #OPTIMIZE  computed ahead of time, stashed, and then fetched to do the actual precompute.
-    #OPTIMIZE
+    #getAllowedResources($tag, "mhap");
 
-    my $cygA       = ($^O ne "cygwin") ? "" : "\$(cygpath -w ";   #  Some baloney to convert nice POSIX paths into
-    my $cygB       = ($^O ne "cygwin") ? "" : ")";                #  whatever abomination Windows expects.
+    my $javaPath = getGlobal("java");
 
     open(F, "> $path/precompute.sh") or caFailure("can't open '$path/precompute.sh' for writing: $!", undef);
 
     print F "#!" . getGlobal("shell") . "\n";
     print F "\n";
-    print F getBinDirectoryShellCode();
-    print F "\n";
-    print F setWorkDirectoryShellCode($path);
-    print F fetchStoreShellCode("$base/$asm.gkpStore", "$base/1-overlapper", "");
-    print F "\n";
-    print F getJobIDShellCode();
+    print F "jobid=\$" . getGlobal("gridEngineTaskID") . "\n";
+    print F "if [ x\$jobid = x -o x\$jobid = xundefined -o x\$jobid = x0 ]; then\n";
+    print F "  jobid=\$1\n";
+    print F "fi\n";
+    print F "if [ x\$jobid = x ]; then\n";
+    print F "  echo Error: I need " . getGlobal("gridEngineTaskID") . " set, or a job index on the command line.\n";
+    print F "  exit 1\n";
+    print F "fi\n";
     print F "\n";
     for (my $ii=1; $ii < scalar(@blocks); $ii++) {
         print F "if [ \$jobid -eq $ii ] ; then\n";
@@ -324,75 +320,67 @@ sub mhapConfigure ($$$) {
     print F "  exit 1\n";
     print F "fi\n";
     print F "\n";
-    print F "if [ ! -d ./blocks ]; then\n";
-    print F "  mkdir -p ./blocks\n";
+    print F "if [ ! -d $path/blocks ]; then\n";
+    print F "  mkdir $path/blocks\n";
     print F "fi\n";
     print F "\n";
-    print F fileExistsShellCode("./blocks/\$job.dat");
+    print F "if [ -e $path/blocks/\$job.dat ]; then\n";
     print F "  echo Job previously completed successfully.\n";
     print F "  exit\n";
     print F "fi\n";
     print F "\n";
-    print F "#  Remove any previous result.\n";
-    print F "rm -f ./blocks/\$job.input.dat\n";
+    print F "#  If the fasta exists, our job failed, and we should try again.\n";
+    print F "if [ -e \"$path/blocks/\$job.fasta\" ] ; then\n";
+    print F "  rm -f $path/blocks/\$job.dat\n";
+    print F "fi\n";
     print F "\n";
-    #print F fetchFileShellCode("./blocks/\$job.input.fasta");
+    print F getBinDirectoryShellCode();
     print F "\n";
     print F "\$bin/gatekeeperDumpFASTQ \\\n";
-    print F "  -G ../$asm.gkpStore \\\n";
+    print F "  -G $wrk/$asm.gkpStore \\\n";
     print F "  \$rge \\\n";
     print F "  -nolibname \\\n";
-    print F "  -noreadname \\\n";
     print F "  -fasta \\\n";
-    print F "  -o ./blocks/\$job.input \\\n";
+    print F "  -o $path/blocks/\$job \\\n";
     print F "|| \\\n";
-    print F "mv -f ./blocks/\$job.input.fasta ./blocks/\$job.input.fasta.FAILED\n";
+    print F "mv -f $path/blocks/\$job.fasta $path/blocks/\$job.fasta.FAILED\n";
     print F "\n";
     print F "\n";
-    print F "if [ ! -e ./blocks/\$job.input.fasta ] ; then\n";
+    print F "if [ ! -e \"$path/blocks/\$job.fasta\" ] ; then\n";
     print F "  echo Failed to extract fasta.\n";
     print F "  exit 1\n";
     print F "fi\n";
     print F "\n";
-    print F fetchFileShellCode("$base/0-mercounts", "$asm.ms$merSize.frequentMers.ignore.gz", "");
-    print F "\n";
-    print F "echo \"\"\n";
     print F "echo Starting mhap precompute.\n";
-    print F "echo \"\"\n";
     print F "\n";
     print F "#  So mhap writes its output in the correct spot.\n";
-    print F "cd ./blocks\n";
+    print F "cd $path/blocks\n";
     print F "\n";
-    print F "$javaPath -d64 -server -Xmx", $javaMemory, "m \\\n";
-    print F "  -jar $cygA \$bin/../share/java/classes/mhap-" . getGlobal("${tag}MhapVersion") . ".jar $cygB \\\n";
-    print F "  --repeat-weight 0.9 --repeat-idf-scale 10 -k $merSize \\\n";
+    print F "$javaPath -d64 -server -Xmx", getGlobal("${tag}mhapMemory"), "g \\\n";
+    print F "  -jar " . ($^O eq "cygwin" ? "\$(cygpath -w " : "") . "\$bin/mhap-" . getGlobal("${tag}MhapVersion") . ".jar " . ($^O eq "cygwin" ? ")" : "") . "\\\n";
+    print F "  --repeat-weight 0.9 -k $merSize \\\n";
     print F "  --supress-noise 2 \\\n"  if (defined(getGlobal("${tag}MhapFilterUnique")) && getGlobal("${tag}MhapFilterUnique") == 1);
     print F "  --no-tf \\\n"            if (defined(getGlobal("${tag}MhapNoTf")) && getGlobal("${tag}MhapNoTf") == 1);
-    print F "  --store-full-id \\\n";
     print F "  --num-hashes $numHashes \\\n";
     print F "  --num-min-matches $minNumMatches \\\n";
     print F "  --ordered-sketch-size $ordSketch \\\n";
     print F "  --ordered-kmer-size $ordSketchMer \\\n";
     print F "  --threshold $threshold \\\n";
     print F "  --filter-threshold $filterThreshold \\\n";
-    print F "  --min-olap-length ", getGlobal("minOverlapLength"), " \\\n";
     print F "  --num-threads ", getGlobal("${tag}mhapThreads"), " \\\n";
-    print F " " . getGlobal("${tag}MhapOptions")         . " \\\n"   if (defined(getGlobal("${tag}MhapOptions")));
-    print F "  -f $cygA ../../0-mercounts/$asm.ms$merSize.frequentMers.ignore.gz $cygB \\\n"   if (-e "$base/0-mercounts/$asm.ms$merSize.frequentMers.ignore.gz");
-    print F "  -p $cygA ./\$job.input.fasta $cygB \\\n";
-    print F "  -q $cygA . $cygB \\\n";
-    print F "&& \\\n";
-    print F "mv -f ./\$job.input.dat ./\$job.dat\n";
+    print F "  -f " . ($^O eq "cygwin" ? "\$(cygpath -w " : "") . "$wrk/0-mercounts/$asm.ms$merSize.frequentMers.ignore.gz" . ($^O eq "cygwin" ? ") " : "") . "\\\n"   if (-e "$wrk/0-mercounts/$asm.ms$merSize.frequentMers.ignore.gz");
+    print F "  -p " . ($^O eq "cygwin" ? "\$(cygpath -w " : "") . "$path/blocks/\$job.fasta" .  ($^O eq "cygwin" ? ") " : "") . "\\\n";
+    print F "  -q " . ($^O eq "cygwin" ? "\$(cygpath -w " : "") . "$path/blocks" .($^O eq "cygwin" ? ") " : "") . "\\\n";
+    print F "|| \\\n";
+    print F "mv -f $path/blocks/\$job.dat $path/blocks/\$job.dat.FAILED\n";
     print F "\n";
-    print F "if [ ! -e ./\$job.dat ] ; then\n";
+    print F "if [ ! -e \"$path/blocks/\$job.dat\" ] ; then\n";
     print F "  echo Mhap failed.\n";
     print F "  exit 1\n";
     print F "fi\n";
     print F "\n";
     print F "#  Clean up, remove the fasta input\n";
-    print F "rm -f ./\$job.input.fasta\n";
-    print F "\n";
-    print F stashFileShellCode("$base/1-overlapper/blocks", "\$job.dat", "");
+    print F "rm -f $path/blocks/\$job.fasta\n";
     print F "\n";
     print F "exit 0\n";
 
@@ -404,17 +392,20 @@ sub mhapConfigure ($$$) {
 
     print F "#!" . getGlobal("shell") . "\n";
     print F "\n";
-    print F getBinDirectoryShellCode();
-    print F "\n";
-    print F setWorkDirectoryShellCode($path);
-    print F fetchStoreShellCode("$base/$asm.gkpStore", "$base/1-overlapper", "");
-    print F "\n";
-    print F getJobIDShellCode();
+    print F "jobid=\$" . getGlobal("gridEngineTaskID") . "\n";
+    print F "if [ x\$jobid = x -o x\$jobid = xundefined -o x\$jobid = x0 ]; then\n";
+    print F "  jobid=\$1\n";
+    print F "fi\n";
+    print F "if [ x\$jobid = x ]; then\n";
+    print F "  echo Error: I need " . getGlobal("gridEngineTaskID") . " set, or a job index on the command line.\n";
+    print F "  exit 1\n";
+    print F "fi\n";
     print F "\n";
     for (my $ii=1; $ii < scalar(@hashes); $ii++) {
         print F "if [ \$jobid -eq $ii ] ; then\n";
         print F "  blk=\"$hashes[$ii]\"\n";
         print F "  slf=\"$skipSelf[$ii]\"\n";
+        print F "  cvt=\"$convert[$ii]\"\n";
         print F "  qry=\"", substr("000000" . $ii, -6), "\"\n";
         print F "fi\n";
         print F "\n";
@@ -426,110 +417,82 @@ sub mhapConfigure ($$$) {
     print F "  exit 1\n";
     print F "fi\n";
     print F "\n";
-    print F "if [ -e ./results/\$qry.ovb ]; then\n";
+    print F "if [ -e $path/results/\$qry.ovb.gz ]; then\n";
     print F "  echo Job previously completed successfully.\n";
     print F "  exit\n";
     print F "fi\n";
     print F "\n";
-
-    print F fetchFileShellCode("$path", "queries.tar", "");
-    print F "\n";
-    print F "if [ -e ./queries.tar -a ! -d ./queries ] ; then\n";
-    print F "  tar -xf ./queries.tar\n";
+    print F "if [ ! -d $path/results ]; then\n";
+    print F "  mkdir $path/results\n";
     print F "fi\n";
     print F "\n";
 
-    print F "if [ ! -d ./results ]; then\n";
-    print F "  mkdir -p ./results\n";
-    print F "fi\n";
-    print F "\n";
-    print F "if [ ! -d ./blocks ] ; then\n";
-    print F "  mkdir -p ./blocks\n";
-    print F "fi\n";
-
-    print F fetchFileShellCode("$path", "blocks/\$blk.dat", "");
-
-    print F "for ii in `ls ./queries/\$qry` ; do\n";
-    print F "  echo Fetch blocks/\$ii\n";
-    print F    fetchFileShellCode("$path", "blocks/\$ii", "  ");
-    print F "done\n";
-    print F "\n";
-
-    print F fetchFileShellCode("$base/0-mercounts", "$asm.ms$merSize.frequentMers.ignore.gz", "");
-    print F "\n";
-
-    print F "echo \"\"\n";
     print F "echo Running block \$blk in query \$qry\n";
-    print F "echo \"\"\n";
+
     print F "\n";
-    print F "if [ ! -e ./results/\$qry.mhap ] ; then\n";
-    print F "  $javaPath -d64 -server -Xmx", $javaMemory, "m \\\n";
-    print F "    -jar $cygA \$bin/../share/java/classes/mhap-" . getGlobal("${tag}MhapVersion") . ".jar $cygB \\\n";
-    print F "    --repeat-weight 0.9 --repeat-idf-scale 10 -k $merSize \\\n";
+    print F getBinDirectoryShellCode();
+    print F "\n";
+    print F "if [ ! -e \"$path/results/\$qry.mhap\" ] ; then\n";
+    print F "  $javaPath -d64 -server -Xmx", getGlobal("${tag}mhapMemory"), "g \\\n";
+    print F "    -jar " . ($^O eq "cygwin" ? "\$(cygpath -w " : "") . "\$bin/mhap-" . getGlobal("${tag}MhapVersion") . ".jar " . ($^O eq "cygwin" ? ")" : "") . "\\\n";
+    print F "    --repeat-weight 0.9 -k $merSize \\\n";
     print F "    --supress-noise 2 \\\n"  if (defined(getGlobal("${tag}MhapFilterUnique")) && getGlobal("${tag}MhapFilterUnique") == 1);
     print F "    --no-tf \\\n"            if (defined(getGlobal("${tag}MhapNoTf")) && getGlobal("${tag}MhapNoTf") == 1);
-    print F "    --store-full-id \\\n";
     print F "    --num-hashes $numHashes \\\n";
     print F "    --num-min-matches $minNumMatches \\\n";
     print F "    --threshold $threshold \\\n";
     print F "    --filter-threshold $filterThreshold \\\n";
     print F "    --ordered-sketch-size $ordSketch \\\n";
     print F "    --ordered-kmer-size $ordSketchMer \\\n";
-    print F "    --min-olap-length ", getGlobal("minOverlapLength"), " \\\n";
     print F "    --num-threads ", getGlobal("${tag}mhapThreads"), " \\\n";
-    print F " " . getGlobal("${tag}MhapOptions")         . " \\\n"   if (defined(getGlobal("${tag}MhapOptions")));
-    print F "    -s $cygA ./blocks/\$blk.dat \$slf $cygB \\\n";
-    print F "    -q $cygA queries/\$qry $cygB \\\n";
-    print F "  > ./results/\$qry.mhap.WORKING \\\n";
+    print F "    -f " . ($^O eq "cygwin" ? "\$(cygpath -w " : "") . "$wrk/0-mercounts/$asm.ms$merSize.frequentMers.ignore.gz" .  ($^O eq "cygwin" ? ")" : "") . "\\\n"   if (-e "$wrk/0-mercounts/$asm.ms$merSize.frequentMers.ignore.gz");
+    print F "    -s " . ($^O eq "cygwin" ? "\$(cygpath -w " : "") . "$path/blocks/\$blk.dat \$slf" .  ($^O eq "cygwin" ? ")" : "") . "\\\n";
+    print F "    -q " . ($^O eq "cygwin" ? "\$(cygpath -w " : "") . "$path/queries/\$qry" . ($^O eq "cygwin" ? ")" : "") . "\\\n";
+    print F "  > $path/results/\$qry.mhap.WORKING \\\n";
     print F "  && \\\n";
-    print F "  mv -f ./results/\$qry.mhap.WORKING ./results/\$qry.mhap\n";
+    print F "  mv -f $path/results/\$qry.mhap.WORKING $path/results/\$qry.mhap\n";
     print F "fi\n";
     print F "\n";
 
-    print F "if [   -e ./results/\$qry.mhap -a \\\n";
-    print F "     ! -e ./results/\$qry.ovb ] ; then\n";
+    print F "if [   -e \"$path/results/\$qry.mhap\" -a \\\n";
+    print F "     ! -e \"$path/results/\$qry.ovb.gz\" ] ; then\n";
     print F "  \$bin/mhapConvert \\\n";
-    print F "    -G ../$asm.gkpStore \\\n";
-    print F "    -o ./results/\$qry.mhap.ovb.WORKING \\\n";
-    print F "    ./results/\$qry.mhap \\\n";
+    print F "    \$cvt \\\n";
+    print F "    -o $path/results/\$qry.mhap.ovb.WORKING.gz \\\n";
+    print F "    $path/results/\$qry.mhap \\\n";
     print F "  && \\\n";
-    print F "  mv ./results/\$qry.mhap.ovb.WORKING ./results/\$qry.mhap.ovb\n";
+    print F "  mv $path/results/\$qry.mhap.ovb.WORKING.gz $path/results/\$qry.mhap.ovb.gz\n";
     print F "fi\n";
     print F "\n";
 
     if (getGlobal('saveOverlaps') eq "0") {
-        print F "if [   -e ./results/\$qry.mhap -a \\\n";
-        print F "       -e ./results/\$qry.mhap.ovb ] ; then\n";
-        print F "  rm -f ./results/\$qry.mhap\n";
+        print F "if [   -e \"$path/results/\$qry.mhap\" -a \\\n";
+        print F "       -e \"$path/results/\$qry.mhap.ovb.gz\" ] ; then\n";
+        print F "  rm -f $path/results/\$qry.mhap\n";
         print F "fi\n";
         print F "\n";
     }
 
-    if (getGlobal("${tag}ReAlign") eq "1") {
-        print F "if [ -e ./results/\$qry.mhap.ovb ] ; then\n";
+    print F "if [ -e \"$path/results/\$qry.mhap.ovb.gz\" ] ; then\n";
+    if (getGlobal("${tag}ReAlign") eq "raw") {
         print F "  \$bin/overlapPair \\\n";
-        print F "    -G ../$asm.gkpStore \\\n";
-        print F "    -O ./results/\$qry.mhap.ovb \\\n";
-        print F "    -o ./results/\$qry.WORKING.ovb \\\n";
+        print F "    -G $wrk/$asm.gkpStore \\\n";
+        print F "    -O $path/results/\$qry.mhap.ovb.gz \\\n";
+        print F "    -o $path/results/\$qry.ovb.gz \\\n";
         print F "    -partial \\\n"  if ($typ eq "partial");
-        print F "    -len "  , getGlobal("minOverlapLength"),  " \\\n";
-        print F "    -erate ", getGlobal("corOvlErrorRate"), " \\\n"  if ($tag eq "cor");   #  Explicitly using proper name for grepability.
+        print F "    -erate ", getGlobal("corErrorRate"),    " \\\n"  if ($tag eq "cor");
         print F "    -erate ", getGlobal("obtOvlErrorRate"), " \\\n"  if ($tag eq "obt");
         print F "    -erate ", getGlobal("utgOvlErrorRate"), " \\\n"  if ($tag eq "utg");
         print F "    -memory " . getGlobal("${tag}mhapMemory") . " \\\n";
-        print F "    -t " . getGlobal("${tag}mhapThreads") . " \\\n";
-        print F "  && \\\n";
-        print F "  mv -f ./results/\$qry.WORKING.ovb ./results/\$qry.ovb\n";
-        print F "fi\n";
+        print F "    -t " . getGlobal("${tag}mhapThreads") . " \n";
     } else {
-        print F "if [ -e ./results/\$qry.mhap.ovb ] ; then\n";
-        print F "  mv -f ./results/\$qry.mhap.ovb ./results/\$qry.ovb\n";
-        print F "fi\n";
+        print F "  mv -f \"$path/results/\$qry.mhap.ovb.gz\" \"$path/results/\$qry.ovb.gz\"\n";
     }
+    print F "fi\n";
 
     print F "\n";
-    print F stashFileShellCode("$path", "results/\$qry.ovb",    "");
-    print F stashFileShellCode("$path", "results/\$qry.counts", "");
+    print F "\n";
+    #print F "rm -rf $path/queries/\$qry\n";
     print F "\n";
     print F "exit 0\n";
 
@@ -550,50 +513,41 @@ sub mhapConfigure ($$$) {
         my $numJobs = 0;
         open(F, "< $path/mhap.sh") or caFailure("can't open '$path/mhap.sh' for reading: $!", undef);
         while (<F>) {
-            $numJobs++  if (m/^\s+qry=/);
+            $numJobs++  if (m/^\s+qry=\"(\d+)\"$/);
         }
         close(F);
 
         print STDERR "-- Configured $numJobs mhap overlap jobs.\n";
     }
 
-    makeExecutable("$path/precompute.sh");
-    makeExecutable("$path/mhap.sh");
-
-    stashFile("$path/precompute.sh");
-    stashFile("$path/mhap.sh");
-
   finishStage:
-    emitStage($asm, "$tag-mhapConfigure");
+    emitStage($WRK, $asm, "$tag-mhapConfigure");
+    buildHTML($WRK, $asm, $tag);
+    stopAfter("mhapConfigure");
 
   allDone:
-    stopAfter("overlapConfigure");
 }
 
 
 
 
-sub mhapPrecomputeCheck ($$$) {
+sub mhapPrecomputeCheck ($$$$) {
+    my $WRK     = shift @_;  #  Root work directory (the -d option to canu)
+    my $wrk     = $WRK;      #  Local work directory
     my $asm     = shift @_;
     my $tag     = shift @_;
     my $typ     = shift @_;
     my $attempt = getGlobal("canuIteration");
 
-    my $base;                #  e.g., $base/1-overlapper/mhap.sh
-    my $path;                #  e.g., $path/mhap.sh
+    $wrk = "$wrk/correction"  if ($tag eq "cor");
+    $wrk = "$wrk/trimming"    if ($tag eq "obt");
+    $wrk = "$wrk/unitigging"  if ($tag eq "utg");
 
-    $base = "correction"  if ($tag eq "cor");
-    $base = "trimming"    if ($tag eq "obt");
-    $base = "unitigging"  if ($tag eq "utg");
+    my $path    = "$wrk/1-overlapper";
 
-    $path = "$base/1-overlapper";
-
-    goto allDone   if (skipStage($asm, "$tag-mhapPrecomputeCheck", $attempt) == 1);
-    goto allDone   if (fileExists("$path/precompute.files"));
-    goto allDone   if (-e "$base/$asm.ovlStore");
-    goto allDone   if (fileExists("$base/$asm.ovlStore.tar"));
-
-    fetchFile("$path/precompute.sh");
+    goto allDone   if (skipStage($WRK, $asm, "$tag-mhapPrecomputeCheck", $attempt) == 1);
+    goto allDone   if (-e "$path/precompute.files");
+    goto allDone   if (-e "$wrk/$asm.ovlStore");
 
     #  Figure out if all the tasks finished correctly.
 
@@ -604,9 +558,9 @@ sub mhapPrecomputeCheck ($$$) {
 
     open(F, "< $path/precompute.sh") or caFailure("can't open '$path/precompute.sh' for reading: $!", undef);
     while (<F>) {
-       if (m/^\s+job=\"(\d+)\"$/) {
-            if (fileExists("$path/blocks/$1.dat")) {
-                push @successJobs, "1-overlapper/blocks/$1.dat\n";
+        if (m/^\s+job=\"(\d+)\"$/) {
+            if (-e "$path/blocks/$1.dat") {
+                push @successJobs, "$path/blocks/$1.dat\n";
             } else {
                 $failureMessage .= "--   job $path/blocks/$1.dat FAILED.\n";
                 push @failedJobs, $currentJobID;
@@ -621,28 +575,29 @@ sub mhapPrecomputeCheck ($$$) {
 
     if (scalar(@failedJobs) > 0) {
 
-        #  If too many attempts, give up.
+        #  If not the first attempt, report the jobs that failed, and that we're recomputing.
 
-        if ($attempt >= getGlobal("canuIterationMax")) {
+        if ($attempt > 1) {
             print STDERR "--\n";
-            print STDERR "-- Mhap precompute jobs failed, tried $attempt times, giving up.\n";
+            print STDERR "-- ", scalar(@failedJobs), " mhap precompute jobs failed:\n";
             print STDERR $failureMessage;
             print STDERR "--\n";
-            caExit(undef, undef);
         }
 
-        if ($attempt > 0) {
-            print STDERR "--\n";
-            print STDERR "-- Mhap precompute jobs failed, retry.\n";
-            print STDERR $failureMessage;
-            print STDERR "--\n";
+        #  If too many attempts, give up.
+
+        if ($attempt > getGlobal("canuIterationMax")) {
+            caExit("failed to precompute mhap indices.  Made " . ($attempt-1) . " attempts, jobs still failed", undef);
         }
 
         #  Otherwise, run some jobs.
 
-        emitStage($asm, "$tag-mhapPrecomputeCheck", $attempt);
+        print STDERR "-- mhap precompute attempt $attempt begins with ", scalar(@successJobs), " finished, and ", scalar(@failedJobs), " to compute.\n";
 
-        submitOrRunParallelJob($asm, "${tag}mhap", $path, "precompute", @failedJobs);
+        emitStage($WRK, $asm, "$tag-mhapPrecomputeCheck", $attempt);
+        buildHTML($WRK, $asm, $tag);
+
+        submitOrRunParallelJob($WRK, $asm, "${tag}mhap", $path, "precompute", @failedJobs);
         return;
     }
 
@@ -653,9 +608,10 @@ sub mhapPrecomputeCheck ($$$) {
     print L @successJobs;
     close(L);
 
-    stashFile("$path/precompute.files");
-
-    emitStage($asm, "$tag-mhapPrecomputeCheck");
+    setGlobal("canuIteration", 0);
+    emitStage($WRK, $asm, "$tag-mhapPrecomputeCheck");
+    buildHTML($WRK, $asm, $tag);
+    stopAfter("mhapPrecompute");
 
   allDone:
 }
@@ -665,27 +621,23 @@ sub mhapPrecomputeCheck ($$$) {
 
 
 
-sub mhapCheck ($$$) {
+sub mhapCheck ($$$$) {
+    my $WRK     = shift @_;  #  Root work directory (the -d option to canu)
+    my $wrk     = $WRK;      #  Local work directory
     my $asm     = shift @_;
     my $tag     = shift @_;
     my $typ     = shift @_;
     my $attempt = getGlobal("canuIteration");
 
-    my $base;                #  e.g., $base/1-overlapper/mhap.sh
-    my $path;                #  e.g., $path/mhap.sh
+    $wrk = "$wrk/correction"  if ($tag eq "cor");
+    $wrk = "$wrk/trimming"    if ($tag eq "obt");
+    $wrk = "$wrk/unitigging"  if ($tag eq "utg");
 
-    $base = "correction"  if ($tag eq "cor");
-    $base = "trimming"    if ($tag eq "obt");
-    $base = "unitigging"  if ($tag eq "utg");
+    my $path    = "$wrk/1-overlapper";
 
-    $path = "$base/1-overlapper";
-
-    goto allDone   if (skipStage($asm, "$tag-mhapCheck", $attempt) == 1);
-    goto allDone   if (fileExists("$path/mhap.files"));
-    goto allDone   if (-e "$base/$asm.ovlStore");
-    goto allDone   if (fileExists("$base/$asm.ovlStore.tar"));
-
-    fetchFile("$path/mhap.sh");
+    goto allDone   if (skipStage($WRK, $asm, "$tag-mhapCheck", $attempt) == 1);
+    goto allDone   if (-e "$path/mhap.files");
+    goto allDone   if (-e "$wrk/$asm.ovlStore");
 
     #  Figure out if all the tasks finished correctly.
 
@@ -699,29 +651,29 @@ sub mhapCheck ($$$) {
     open(F, "< $path/mhap.sh") or caExit("failed to open '$path/mhap.sh'", undef);
     while (<F>) {
         if (m/^\s+qry=\"(\d+)\"$/) {
-            if      (fileExists("$path/results/$1.ovb.gz")) {
-                push @mhapJobs,    "1-overlapper/results/$1.mhap\n";
-                push @successJobs, "1-overlapper/results/$1.ovb.gz\n";
-                push @miscJobs,    "1-overlapper/results/$1.stats\n";
-                push @miscJobs,    "1-overlapper/results/$1.counts\n";
+            if      (-e "$path/results/$1.ovb.gz") {
+                push @mhapJobs,    "$path/results/$1.mhap\n";
+                push @successJobs, "$path/results/$1.ovb.gz\n";
+                push @miscJobs,    "$path/results/$1.stats\n";
+                push @miscJobs,    "$path/results/$1.counts\n";
 
-            } elsif (fileExists("$path/results/$1.ovb")) {
-                push @mhapJobs,    "1-overlapper/results/$1.mhap\n";
-                push @successJobs, "1-overlapper/results/$1.ovb\n";
-                push @miscJobs,    "1-overlapper/results/$1.stats\n";
-                push @miscJobs,    "1-overlapper/results/$1.counts\n";
+            } elsif (-e "$path/results/$1.ovb") {
+                push @mhapJobs,    "$path/results/$1.mhap\n";
+                push @successJobs, "$path/results/$1.ovb\n";
+                push @miscJobs,    "$path/results/$1.stats\n";
+                push @miscJobs,    "$path/results/$1.counts\n";
 
-            } elsif (fileExists("$path/results/$1.ovb.bz2")) {
-                push @mhapJobs,    "1-overlapper/results/$1.mhap\n";
-                push @successJobs, "1-overlapper/results/$1.ovb.bz2\n";
-                push @miscJobs,    "1-overlapper/results/$1.stats\n";
-                push @miscJobs,    "1-overlapper/results/$1.counts\n";
+            } elsif (-e "$path/results/$1.ovb.bz2") {
+                push @mhapJobs,    "$path/results/$1.mhap\n";
+                push @successJobs, "$path/results/$1.ovb.bz2\n";
+                push @miscJobs,    "$path/results/$1.stats\n";
+                push @miscJobs,    "$path/results/$1.counts\n";
 
-            } elsif (fileExists("$path/results/$1.ovb.xz")) {
-                push @mhapJobs,    "1-overlapper/results/$1.mhap\n";
-                push @successJobs, "1-overlapper/results/$1.ovb.xz\n";
-                push @miscJobs,    "1-overlapper/results/$1.stats\n";
-                push @miscJobs,    "1-overlapper/results/$1.counts\n";
+            } elsif (-e "$path/results/$1.ovb.xz") {
+                push @mhapJobs,    "$path/results/$1.mhap\n";
+                push @successJobs, "$path/results/$1.ovb.xz\n";
+                push @miscJobs,    "$path/results/$1.stats\n";
+                push @miscJobs,    "$path/results/$1.counts\n";
 
             } else {
                 $failureMessage .= "--   job $path/results/$1.ovb FAILED.\n";
@@ -736,7 +688,7 @@ sub mhapCheck ($$$) {
     #  Also find the queries symlinks so we can remove those.  And the query directories, because
     #  the last directory can be empty, and so we'd never see it at all if only finding files.
 
-    open(F, "cd $base && find 1-overlapper/queries -print |");
+    open(F, "find $path/queries -print |");
     while (<F>) {
         push @mhapJobs, $_;
     }
@@ -746,27 +698,28 @@ sub mhapCheck ($$$) {
 
     if (scalar(@failedJobs) > 0) {
 
-        #  If too many attempts, give up.
+        #  If not the first attempt, report the jobs that failed, and that we're recomputing.
 
-        if ($attempt >= getGlobal("canuIterationMax")) {
+        if ($attempt > 1) {
             print STDERR "--\n";
-            print STDERR "-- Mhap overlap jobs failed, tried $attempt times, giving up.\n";
+            print STDERR "-- ", scalar(@failedJobs), " mhap jobs failed:\n";
             print STDERR $failureMessage;
             print STDERR "--\n";
-            caExit(undef, undef);
         }
 
-        if ($attempt > 0) {
-            print STDERR "--\n";
-            print STDERR "-- Mhap overlap jobs failed, retry.\n";
-            print STDERR $failureMessage;
-            print STDERR "--\n";
+        #  If too many attempts, give up.
+
+        if ($attempt > getGlobal("canuIterationMax")) {
+            caExit("failed to compute mhap overlaps.  Made " . ($attempt-1) . " attempts, jobs still failed", undef);
         }
 
         #  Otherwise, run some jobs.
 
-        emitStage($asm, "$tag-mhapCheck", $attempt);
-        submitOrRunParallelJob($asm, "${tag}mhap", $path, "mhap", @failedJobs);
+        print STDERR "-- mhap attempt $attempt begins with ", scalar(@successJobs), " finished, and ", scalar(@failedJobs), " to compute.\n";
+
+        emitStage($WRK, $asm, "$tag-mhapCheck", $attempt);
+        buildHTML($WRK, $asm, $tag);
+        submitOrRunParallelJob($WRK, $asm, "${tag}mhap", $path, "mhap", @failedJobs);
         return;
     }
 
@@ -785,13 +738,11 @@ sub mhapCheck ($$$) {
     print L @miscJobs;
     close(L);
 
-    stashFile("$path/mhap.files");
-    stashFile("$path/ovljob.files");
-    stashFile("$path/ovljob.more.files");
-
-    emitStage($asm, "$tag-mhapCheck");
+    setGlobal("canuIteration", 0);
+    emitStage($WRK, $asm, "$tag-mhapCheck");
+    buildHTML($WRK, $asm, $tag);
+    stopAfter("mhapOverlap");
 
   allDone:
-    stopAfter("overlap");
 }
 
